package repository

import (
	"fmt"
	"github.com/pkg/errors"
	"sghgogs.com/micro/shopping-service-service/domain/model/base"
	req "sghgogs.com/micro/shopping-service-service/domain/model/request"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"time"
)

// GetOrderList 获取列表
func (u *Repository) GetOrderList(query *pb.GetOrderListRequest) ([]req.Order, int64, error) {
	tx := u.db.Model(req.Order{}).
		Select("id, user_id, address_id, total_amount, status, payment_method, created_at, updated_at, approver_id").
		Preload("User").
		Preload("Approver").
		Preload("Address").
		Order("id desc")
	fmt.Println(base.IsValidOrderStatus(query.Status))
	if base.IsValidOrderStatus(query.Status) {
		tx.Where("status = ?", query.Status)
	}
	if base.IsValidPaymentMethod(query.PaymentMethod) {
		tx.Where("payment_method = ? ", query.PaymentMethod)
	}
	var totalCount int64
	tx.Count(&totalCount)
	orderList := make([]req.Order, 0)
	return orderList, totalCount, tx.Limit(int(query.PageSize)).Offset(int((query.Page - 1) * query.PageSize)).Find(&orderList).Error
}

func (u *Repository) GetOrder(orderID int64) (req.Order, error) {
	var order req.Order
	return order, u.db.Model(&req.Order{}).
		Where("id = ?", orderID).
		Preload("OrderItems").
		Preload("OrderItems.Product").
		Preload("OrderItems.Product.Category").
		Preload("Address").
		Preload("User").
		Preload("Approver").
		Preload("Payment").
		First(&order).Error
}

// ConfirmOrderProcessing 处理中 商家已确认订单，并开始处理订单中的商品。
func (u *Repository) ConfirmOrderProcessing(orderID, approverID int64) error {
	// 开始事务
	tx := u.db.Begin()
	// 错误处理
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()
	var order req.Order
	if err := tx.Model(&req.Order{}).Where("id = ?", orderID).First(&order).Error; err != nil {
		tx.Rollback()
		return err
	} else {
		if order.Status != pb.OrderStatusEnum_ORDER_PAID {
			tx.Rollback()
			return errors.New("Order cannot be reviewed in the current state. Please ensure that the order has been successfully paid for and is in a reviewable state.")
		}
	}
	if err := tx.Model(&order).Updates(map[string]interface{}{
		"status":      pb.OrderStatusEnum_ORDER_PROCESSING,
		"approver_id": approverID,
		"updated_at":  time.Now(),
	}).Error; err != nil {
		tx.Rollback()
		return err
	}
	return tx.Commit().Error
}

// InitiateShipping 发货中 商品已从商家发货，运送中
func (u *Repository) InitiateShipping(orderID, approverID int64) error {
	// 开始事务
	tx := u.db.Begin()
	// 错误处理
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()
	var order req.Order
	if err := tx.Model(&req.Order{}).Where("id = ?", orderID).First(&order).Error; err != nil {
		tx.Rollback()
		return err
	} else {
		if order.Status != pb.OrderStatusEnum_ORDER_PROCESSING {
			tx.Rollback()
			return errors.New("Order cannot be reviewed in the current state. Please ensure that the order has been successfully paid for and is in a reviewable state.")
		}
	}
	if err := tx.Model(&order).Updates(map[string]interface{}{
		"status":      pb.OrderStatusEnum_ORDER_SHIPPED,
		"approver_id": approverID,
		"updated_at":  time.Now(),
	}).Error; err != nil {
		tx.Rollback()
		return err
	}
	return tx.Commit().Error
}

func (u *Repository) CompleteDelivery(orderID, approverID int64) error {
	// 开始事务
	tx := u.db.Begin()
	// 错误处理
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()
	var order req.Order
	if err := tx.Model(&req.Order{}).Where("id = ?", orderID).First(&order).Error; err != nil {
		tx.Rollback()
		return err
	} else {
		fmt.Println("order.Status", order.Status)
		if order.Status != pb.OrderStatusEnum_ORDER_SHIPPED {
			tx.Rollback()
			return errors.New("Order cannot be reviewed in the current state. Please ensure that the order has been successfully paid for and is in a reviewable state.")
		}
	}
	if err := tx.Model(&order).Updates(map[string]interface{}{
		"status":      pb.OrderStatusEnum_ORDER_COMPLETED,
		"approver_id": approverID,
		"updated_at":  time.Now(),
	}).Error; err != nil {
		tx.Rollback()
		return err
	}
	return tx.Commit().Error
}
