package repository

import (
	"github.com/pkg/errors"
	"gorm.io/gorm"
	req "sghgogs.com/micro/shopping-service-service/domain/model/request"
	pb "sghgogs.com/micro/shopping-service-service/proto"
)

func (u *Repository) IsProductExists(identifier interface{}) (bool, error) {
	var product req.Product
	if err := u.db.
		Where("id = ? OR name = ?", identifier, identifier).
		Select("id, name").
		First(&product).Error; err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return false, nil // 记录不存在，账号不存在
		}
		return false, err // 发生其他错误
	}
	return true, nil
}

func (u *Repository) GetProductList(query *pb.GetProductListRequest) ([]req.Product, int64, error) {
	products := make([]req.Product, 0)
	tx := u.db.Model(&req.Product{}).
		Select("id, name, description, price, stock_quantity, created_at, created_by, updated_at, updated_by, category_id, image_url").
		Order("id desc")
	if query.Keyword != "" {
		tx.Where("name = ? ", query.Keyword)
	}
	if query.CategoryId > 0 {
		tx.Where("category_id = ?", query.CategoryId)
	}
	var totalCount int64
	tx.Count(&totalCount)
	return products, totalCount, tx.Limit(int(query.PageSize)).Offset(int((query.Page - 1) * query.PageSize)).Find(&products).Error
}

func (u *Repository) GetProduct(productID int64) (req.Product, error) {
	var product req.Product
	return product, u.db.Model(&req.Product{}).Preload("Category").Find(&product, productID).Error
}

func (u *Repository) IncreaseProductInventory(productID int64, quantity int32) error {
	var product req.Product
	if err := u.db.First(&product, productID).Error; err != nil {
		return err
	}
	// 检查库存是否充足
	if product.StockQuantity < quantity {
		return errors.New("insufficient stock")
	}
	// 减少库存量
	product.StockQuantity += quantity

	// 更新产品库存
	if err := u.db.Save(&product).Error; err != nil {
		return err
	}

	return nil
}

func (u *Repository) CreateMultipleProducts(products []req.Product) error {
	// 开始事务
	tx := u.db.Begin()
	// 错误处理
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()
	if err := tx.Create(&products).Error; err != nil {
		tx.Rollback()
		return err
	}
	return tx.Commit().Error
}

func (u *Repository) UpdateProduct(productID int64, data map[string]interface{}) error {
	// 开始事务
	tx := u.db.Begin()
	// 错误处理
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()
	var product req.Product
	if err := tx.Model(&req.Product{}).First(&product, productID).Error; err != nil {
		tx.Rollback()
		return err
	}
	if err := tx.Model(&product).Updates(data).Error; err != nil {
		tx.Rollback()
		return err
	}
	return tx.Commit().Error
}

func (u *Repository) DeleteProduct(ProductId int64) error {
	return u.db.Model(&req.Product{}).Delete(&req.Product{ID: ProductId}).Error
}
