package repository

import (
	"fmt"
	"sghgogs.com/micro/shopping-service-service/domain/model/base"
	req "sghgogs.com/micro/shopping-service-service/domain/model/request"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"time"
)

func (u *Repository) GetRefundRequestList(query *pb.GetRefundRequestListRequest) ([]req.RefundRequest, int64, error) {
	var totalCount int64
	tx := u.db.Model(&req.RefundRequest{}).Order("id desc")
	fmt.Println("base.IsValidRefundRequestStatus(query.Status)", base.IsValidRefundRequestStatus(query.Status))
	if base.IsValidRefundRequestStatus(query.Status) {
		tx.Where("status = ?", query.Status)
	}
	tx.Count(&totalCount)
	refundRequests := make([]req.RefundRequest, 0)
	return refundRequests, totalCount, tx.Limit(int(query.PageSize)).Offset(int((query.Page - 1) * query.PageSize)).Find(&refundRequests).Error
}

// CompleteRefund 完成
func (u *Repository) CompleteRefund(refundRequestID int64) error {
	// 开始事务
	tx := u.db.Begin()
	// 错误处理
	defer func() {
		if r := recover(); r != nil {
			tx.Rollback()
		}
	}()
	var refundRequest req.RefundRequest
	if err := tx.Model(req.RefundRequest{}).Where("id = ?", refundRequestID).Where(&refundRequest).Error; err != nil {
		tx.Rollback()
		return err
	}
	// 查询订单
	var order req.Order
	if err := tx.Model(&req.Order{}).Preload("OrderItems").Where("id = ?", refundRequest.OrderID).First(&order).Error; err != nil {
		tx.Rollback()
		return err
	}
	now := time.Now()
	// 退款记录表
	record := req.RefundRecord{
		OrderID:      refundRequest.OrderID,
		Amount:       order.TotalAmount,
		Status:       pb.RefundRecordStatusEnum_REFUND_RECORD_SUCCESS,
		RefundMethod: order.PaymentMethod,
		RefundTime:   &now,
		CreatedAt:    now,
	}
	if err := tx.Create(&record).Error; err != nil {
		tx.Rollback()
		return err
	}
	// 修改库存
	for _, item := range order.OrderItems {
		if err := u.IncreaseProductInventory(item.ProductID, item.Quantity); err != nil {
			tx.Rollback()
			return err
		}
	}
	// 退返金额
	var userAccount req.UserAccount
	if err := tx.Model(req.UserAccount{}).Where("user_id = ?", refundRequest.UserID).First(&userAccount).Error; err != nil {
		tx.Rollback()
		return err
	}
	userAccount.Balance += order.TotalAmount
	if err := tx.Save(&userAccount).Error; err != nil {
		tx.Rollback()
		return err
	}
	// 更新账单信息
	var bill req.Bill
	if err := tx.Model(&req.Bill{}).Where("reference_order_id = ?", refundRequest.OrderID).First(&bill).Error; err != nil {
		tx.Rollback()
		return err
	}
	bill.Type = pb.BillTypeEnum_BILL_REFUND // 退款
	bill.Amount = -bill.Amount
	if err := tx.Model(&req.Bill{}).Where("id = ?", bill.ID).Updates(map[string]interface{}{
		"type":       pb.BillTypeEnum_BILL_REFUND,
		"amount":     -bill.Amount,
		"updated_at": time.Now(),
	}).Error; err != nil {
		tx.Rollback()
		return err
	}
	// 更新支付状态
	var payment req.Payment
	if err := tx.Model(&req.Payment{}).Where("order_id = ?", refundRequest.OrderID).First(&payment).Error; err != nil {
		tx.Rollback()
		return err
	}
	if err := tx.Model(&req.Payment{}).Where("id = ?", payment.ID).Updates(map[string]interface{}{
		"type":       pb.PaymentStatusEnum_PAYMENT_REFUNDED,
		"updated_at": time.Now(),
	}).Error; err != nil {
		tx.Rollback()
		return err
	}
	// 提交事务
	return tx.Commit().Error
}

// ApproveRefund 处理中 拒绝
func (u *Repository) ApproveRefund(refundRequestID int64, status *pb.RefundRequestStatusEnum) error {
	var refundRequest req.RefundRequest
	if err := u.db.Model(req.RefundRequest{}).Where("id = ?", refundRequestID).Where(&refundRequest).Error; err != nil {
		return err
	}
	if !base.IsValidRefundRequestStatus(*status) && *status != pb.RefundRequestStatusEnum_REFUND_REQUEST_STATUS_PROCESSED {
		return fmt.Errorf("cannot update refund request status from %s to %s: %w", refundRequest.Status, status, "invalid refund request status")
	}
	refundRequest.Status = *status
	if err := u.db.Save(&refundRequest).Error; err != nil {
		return err
	}
	return nil
}
