package service

import (
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"net/http"
	"sghgogs.com/micro/common/errorcode"
	req "sghgogs.com/micro/shopping-service-service/domain/model/request"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"sghgogs.com/micro/shopping-service-service/utils"
	"time"
)

func (svc *Service) GetAllCategories() ([]*pb.Category, error) {
	categories := make([]*pb.Category, 0)
	list, err := svc.Repository.GetAllCategories()
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return categories, nil // 记录不存在，角色不存在
		} else {
			return categories, err
		}
	}
	return svc.getCategoryRes(list, false), nil
}

func (svc *Service) GetCategoryList(query *pb.GetCategoryListRequest) ([]*pb.Category, int64, error) {
	categories := make([]*pb.Category, 0)
	list, count, err := svc.Repository.GetCategoryList(query)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return categories, 0, nil // 记录不存在，角色不存在
		} else {
			return categories, 0, err
		}
	}
	return svc.getCategoryRes(list, true), count, nil
}

func (svc *Service) CreateMultipleCategories(category *pb.CreateMultipleCategoriesRequest) error {
	var res req.Category
	if category.ParentCategoryId > 0 {
		resCategory, err := svc.Repository.GetCategoryByID(category.ParentCategoryId)
		if err != nil {
			return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
		}
		res = *resCategory
	}
	if err := svc.Repository.CreateMultipleCategories(svc.getMultipleCategoriesReq(category.ParentCategoryId, res, category)); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
	}
	return nil
}

func (svc *Service) CreateCategory(data *pb.CreateCategoryRequest) error {
	category := req.Category{
		Name:             data.Name,
		Description:      data.Description,
		ParentCategoryID: data.ParentCategoryId,
		CreatedAt:        time.Now(),
		ImageURL:         data.ImageUrl,
		Level:            1,
	}
	if data.ParentCategoryId > 0 {
		resCategory, err := svc.Repository.GetCategoryByID(data.ParentCategoryId)
		if err != nil {
			return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
		}
		category.Level = resCategory.Level + 1
	}
	if err := svc.Repository.CreateCategory(&category); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
	}
	return nil
}

func (svc *Service) DeleteCategory(query *pb.DeleteCategoryRequest) error {
	if err := svc.Repository.DeleteCategory(query.CategoryId); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
	}
	return nil
}

func (svc *Service) getCategoryRes(items []req.Category, isTime bool) []*pb.Category {
	categories := make([]*pb.Category, 0)
	for _, item := range items {
		categories = append(categories, svc.getCategoryItem(&item, isTime))
	}
	return categories
}

func (svc *Service) getCategoryItem(category *req.Category, isTime bool) *pb.Category {
	p := pb.Category{
		Id:               category.ID,
		Name:             category.Name,
		Description:      category.Description,
		ParentCategoryId: category.ParentCategoryID,
		Level:            category.Level,
		ImageUrl:         category.ImageURL,
	}
	if isTime {
		var updatedAt int64
		if category.UpdatedAt != nil {
			if category.CreatedAt.Truncate(time.Second) == category.UpdatedAt.Truncate(time.Second) {
				category.UpdatedAt = nil
			} else {
				updatedAt = utils.ConvertTimeToInt64(*category.UpdatedAt)
			}
		}
		p.CreatedAt = utils.ConvertTimeToInt64(category.CreatedAt)
		p.UpdatedAt = updatedAt
	}
	return &p
}

// resCategory
func (svc *Service) getMultipleCategoriesReq(parentID int64, category req.Category, categories *pb.CreateMultipleCategoriesRequest) []req.Category {
	items := make([]req.Category, 0)
	for _, item := range categories.Items {
		reqItem := svc.getMultipleCategoriesReqItem(item)
		reqItem.ParentCategoryID = parentID
		if parentID > 0 {
			reqItem.Level = category.Level + 1
		} else {
			reqItem.Level = 1
		}
		items = append(items, reqItem)
	}
	return items
}

func (svc *Service) getMultipleCategoriesReqItem(item *pb.CreateMultipleCategoriesItem) req.Category {
	return req.Category{
		Name:        item.Name,
		CreatedAt:   time.Now(),
		Description: item.Description,
		ImageURL:    item.ImageUrl,
		UpdatedAt:   nil,
	}
}
