package service

import (
	"fmt"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"net/http"
	"sghgogs.com/micro/common"
	"sghgogs.com/micro/common/errorcode"
	req "sghgogs.com/micro/shopping-service-service/domain/model/request"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"sghgogs.com/micro/shopping-service-service/utils"
	"time"
)

func (svc *Service) GetOrderList(query *pb.GetOrderListRequest) ([]*pb.Order, int64, error) {
	orders := make([]*pb.Order, 0)
	list, total, err := svc.Repository.GetOrderList(query)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return orders, 0, nil // 记录不存在
		} else {
			return orders, 0, err
		}
	}
	return svc.getOrderResItems(list), total, nil
}

func (svc *Service) GetOrder(query *pb.GetOrderRequest) (*pb.Order, error) {
	order, err := svc.Repository.GetOrder(query.OrderId)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return &pb.Order{}, errorcode.New(svc.Namespace, common.ErrorMessage[common.InvalidOrderIDError], 400)
		} else {
			return &pb.Order{}, err
		}
	}
	for _, item := range order.OrderItems {
		fmt.Println(fmt.Sprintf("GetOrder %+v", item.Product.Category))
	}
	item := svc.getOrderResItem(order)
	item.OrderItems = svc.getOrderItemsRes(order.OrderItems)
	return item, nil
}

func (svc *Service) ConfirmOrderProcessing(data *pb.ConfirmOrderProcessingRequest) error {
	if err := svc.Repository.ConfirmOrderProcessing(data.OrderId, data.ApproverId); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
	}
	return nil
}

func (svc *Service) InitiateShipping(query *pb.InitiateShippingRequest) error {
	if err := svc.Repository.InitiateShipping(query.OrderId, query.ApproverId); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
	}
	return nil
}

func (svc *Service) CompleteDelivery(data *pb.CompleteDeliveryRequest) error {
	if err := svc.Repository.CompleteDelivery(data.OrderId, data.ApproverId); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusInternalServerError)
	}
	return nil
}

func (svc *Service) getOrderResItems(items []req.Order) []*pb.Order {
	orders := make([]*pb.Order, 0)
	for _, item := range items {
		orders = append(orders, svc.getOrderResItem(item))
	}
	return orders
}

func (svc *Service) getOrderResItem(order req.Order) *pb.Order {
	var updatedAt int64
	if order.UpdatedAt != nil {
		if order.CreatedAt.Truncate(time.Second) == order.UpdatedAt.Truncate(time.Second) {
			order.UpdatedAt = nil
		} else {
			updatedAt = utils.ConvertTimeToInt64(*order.UpdatedAt)
		}
	}
	return &pb.Order{
		Id:            order.ID,
		UserId:        order.UserID,
		User:          svc.getUserResItem(&order.User),
		Address:       svc.getAddressResItem(order.Address),
		TotalAmount:   order.TotalAmount,
		Status:        order.Status,
		PaymentMethod: order.PaymentMethod,
		CreatedAt:     utils.ConvertTimeToInt64(order.CreatedAt),
		UpdatedAt:     updatedAt,
		Approver:      svc.getUserResItem(&order.Approver),
	}
}
func (svc *Service) getAddressResItem(address req.Address) *pb.Address {
	var updatedAt int64
	if address.UpdatedAt != nil {
		if address.CreatedAt.Truncate(time.Second) == address.UpdatedAt.Truncate(time.Second) {
			address.UpdatedAt = nil
		} else {
			updatedAt = utils.ConvertTimeToInt64(*address.UpdatedAt)
		}
	}
	location := address.Location
	var item *pb.Location
	if location != nil {
		item = &pb.Location{
			Id:        location.ID,
			AddressId: location.AddressID,
			Lon:       location.Lon,
			Lat:       location.Lat,
		}
	}
	return &pb.Address{
		Id:          address.ID,
		UserId:      address.UserID,
		Name:        address.Name,
		PhoneNumber: address.PhoneNumber,
		CountryCode: address.CountryCode,
		ProvinceId:  address.ProvinceID,
		CityId:      address.CityID,
		DistrictId:  address.DistrictID,
		StreetId:    address.StreetID,
		Remark:      address.Remark,
		IsDefault:   address.IsDefault,
		Location:    item,
		CreatedAt:   utils.ConvertTimeToInt64(address.CreatedAt),
		UpdatedAt:   updatedAt,
	}
}

func (svc *Service) getOrderItemsRes(orders []req.OrderItem) []*pb.OrderItem {
	items := make([]*pb.OrderItem, 0)
	for _, item := range orders {
		items = append(items, svc.getOrderItemRes(item))
	}
	return items
}

func (svc *Service) getOrderItemRes(item req.OrderItem) *pb.OrderItem {
	var updatedAt int64
	if item.UpdatedAt != nil {
		if item.CreatedAt.Truncate(time.Second) == item.UpdatedAt.Truncate(time.Second) {
			item.UpdatedAt = nil
		} else {
			updatedAt = utils.ConvertTimeToInt64(*item.UpdatedAt)
		}
	}
	orderItem := pb.OrderItem{
		Id:        item.ID,
		OrderId:   item.OrderID,
		ProductId: item.ProductID,
		Quantity:  item.Quantity,
		Subtotal:  item.Subtotal,
		CreatedAt: utils.ConvertTimeToInt64(item.CreatedAt),
		UpdatedAt: updatedAt,
	}
	if item.Product != nil {
		orderItem.Product = svc.getProduct(*item.Product)
	}
	return &orderItem
}
