package service

import (
	"context"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"net/http"
	"sghgogs.com/micro/common/errorcode"
	req "sghgogs.com/micro/shopping-service-service/domain/model/request"
	pb "sghgogs.com/micro/shopping-service-service/proto"
	"sghgogs.com/micro/shopping-service-service/utils"
	"time"
)

func (svc *Service) GetProductList(query *pb.GetProductListRequest) ([]*pb.Product, int64, error) {
	products := make([]*pb.Product, 0)

	list, count, err := svc.Repository.GetProductList(query)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return products, 0, nil // 记录不存在，角色不存在
		} else {
			return products, 0, err
		}
	}
	return svc.getProductRes(list), count, nil
}

func (svc *Service) GetProduct(query *pb.GetProductRequest) (*pb.Product, error) {
	product, err := svc.Repository.GetProduct(query.ProductId)
	item := svc.getProductResItem(product)
	item.Category = svc.getCategory(*product.Category)
	return item, err
}

func (svc *Service) CreateMultipleProducts(ctx context.Context, products *pb.CreateProductRequest) error {
	_, by, _, _ := utils.ParseMetadata(ctx)
	if err := svc.Repository.CreateMultipleProducts(svc.getProductsReq(by, products.Items)); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusBadRequest)
	}
	return nil
}

func (svc *Service) UpdateProduct(ctx context.Context, data *pb.UpdateProductRequest) error {
	_, by, _, _ := utils.ParseMetadata(ctx)
	m := map[string]interface{}{
		"name":           data.Name,
		"description":    data.Description,
		"price":          data.Price,
		"stock_quantity": data.StockQuantity,
		"category_id":    data.CategoryId,
		"updated_at":     time.Now(),
		"image_url":      data.ImageUrl,
		"updated_by":     by,
	}
	if err := svc.Repository.UpdateProduct(data.ProductId, m); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusBadRequest)
	}
	return nil
}

func (svc *Service) DeleteProduct(query *pb.DeleteProductRequest) error {
	if err := svc.Repository.DeleteProduct(query.ProductId); err != nil {
		return errorcode.New(svc.Namespace, err.Error(), http.StatusBadRequest)
	}
	return nil
}

func (svc *Service) getProductsReq(by string, items []*pb.Product) []req.Product {
	products := make([]req.Product, 0)
	for _, item := range items {
		products = append(products, req.Product{
			Name:          item.Name,
			Description:   item.Description,
			Price:         item.Price,
			StockQuantity: item.StockQuantity,
			CreatedAt:     time.Now(),
			CreatedBy:     by,
			CategoryID:    item.CategoryId,
			ImageURL:      item.ImageUrl,
		})
	}
	return products
}

func (svc *Service) getProductRes(items []req.Product) []*pb.Product {
	products := make([]*pb.Product, 0)
	for _, item := range items {
		products = append(products, svc.getProductResItem(item))
	}
	return products
}
func (svc *Service) getProductResItem(item req.Product) *pb.Product {
	var updatedAt int64
	if item.UpdatedAt != nil {
		if item.CreatedAt.Truncate(time.Second) == item.UpdatedAt.Truncate(time.Second) {
			item.UpdatedAt = nil
		} else {
			updatedAt = utils.ConvertTimeToInt64(*item.UpdatedAt)
		}
	}
	return &pb.Product{
		Id:            item.ID,
		Name:          item.Name,
		Description:   item.Description,
		Price:         item.Price,
		StockQuantity: item.StockQuantity,
		CategoryId:    item.CategoryID,
		CreatedAt:     utils.ConvertTimeToInt64(item.CreatedAt),
		CreatedBy:     item.CreatedBy,
		UpdatedAt:     updatedAt,
		UpdatedBy:     item.UpdatedBy,
		ImageUrl:      item.ImageURL,
	}
}
