package utils

import (
	"context"
	"fmt"
	"github.com/sirupsen/logrus"
	"go-micro.dev/v4/auth"
	"go-micro.dev/v4/metadata"
	"golang.org/x/crypto/bcrypt"
	"sghgogs.com/micro/common"
	"sghgogs.com/micro/common/errorcode"
	"sghgogs.com/micro/shopping-service-service/utils/authutil"
	"strconv"
	"strings"
	"time"
)

var (
	AdminUserGroupID int64 = 1
	UserUserGroupID  int64 = 2
	UserGroup              = []int64{1, 2}
)

func IsUserGroup(ID int64) bool {
	for _, value := range UserGroup {
		if value == ID {
			return true
		}
	}
	return false
}

func ConvertTimeToInt64(t time.Time) int64 {
	return t.Unix()
}
func ConvertInt64ToTime(seconds int64) time.Time {
	return time.Unix(seconds, 0)
}

// HashPassword 使用 bcrypt 对密码进行哈希
func HashPassword(password string) (string, error) {
	hash, err := bcrypt.GenerateFromPassword([]byte(password), bcrypt.DefaultCost)
	if err != nil {
		return "", err
	}
	return string(hash), nil
}

// CheckPasswordHash 验证用户输入的密码是否与存储的哈希值匹配
func CheckPasswordHash(password, hash string) bool {
	err := bcrypt.CompareHashAndPassword([]byte(hash), []byte(password))
	return err == nil
}

func ParseMetadata(ctx context.Context) (int64, string, string, string) {
	md, b := metadata.FromContext(ctx)
	if !b {
		return 0, "", "", ""
	}
	authHeader, ok := md["Authorization"]
	if !ok || !strings.HasPrefix(authHeader, auth.BearerScheme) {
	}
	token := strings.TrimPrefix(authHeader, auth.BearerScheme)
	inspect, _ := authutil.JWTAuthService.Inspect(token)
	fmt.Println("inspect", inspect)
	var userId int64
	if id, isOk := inspect.Metadata["user_id"]; isOk {
		ID, _ := strconv.ParseInt(id, 0, 64)
		userId = ID
	}
	createdBy := ""
	if username, isOk := inspect.Metadata["username"]; isOk {
		createdBy = username
	}
	IPAddress := ""
	if len(md["Local"]) > 0 {
		IPAddress = md["Local"]
	} else {
		IPAddress = md["Remote"]
	}
	return userId, createdBy, IPAddress, md["Agent"]
}

func LoadMetadata(ctx context.Context, namespace string) (map[string]string, error) {
	md, b := metadata.FromContext(ctx)
	var data map[string]string
	if !b {
		return data, errorcode.Unauthorized(namespace, common.ErrorMessage[common.UnauthorizedErrorCode])
	}
	authHeader, ok := md["Authorization"]

	if !ok || !strings.HasPrefix(authHeader, auth.BearerScheme) {
		logrus.Error("no auth token provided")
		return data, errorcode.Unauthorized(namespace, common.ErrorMessage[common.UnauthorizedErrorCode])
	}
	// Extract auth token.
	token := strings.TrimPrefix(authHeader, auth.BearerScheme)

	token = strings.TrimSpace(token)
	inspect, errE := authutil.JWTAuthService.Inspect(token)
	if errE != nil {
		return data, errorcode.Unauthorized(namespace, common.ErrorMessage[common.TokenInvalidErrorCode])
	}
	return inspect.Metadata, nil
}
